# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from pyVmomi import vim


def get_net_item(host, object_type, name):
    """
    Retrieves a network object of the specified type and name from a host
    :param host: vim.HostSystem
    :param object_type: Type of object to get: (portgroup | vswitch | proxyswitch | vnic | pnic)
    :param name: Name of network object [default: first object found]
    :return: The network object
    """
    if name:
        return get_net_obj(host, object_type, name)
    else:
        return get_net_objs(host, object_type)[0]


def get_net_obj(host, object_type, name, refresh=False):
    """
    Retrieves a network object of the specified type and name from a host
    :param host: vim.HostSystem
    :param object_type: Type of object to get: (portgroup | vswitch | proxyswitch | vnic | pnic)
    :param name: Name of network object
    :param refresh: Refresh the host's network system information [default: False]
    :return: The network object
    """
    objs = get_net_objs(host=host, object_type=object_type, refresh=refresh)
    obj_name = name.lower()
    if objs is not None:
        for obj in objs:
            if object_type == "portgroup" or object_type == "proxyswitch":
                if obj.spec.name.lower() == obj_name:
                    return obj
            elif object_type == "pnic" or object_type == "vnic":
                if obj.device.lower() == obj_name:
                    return obj
            elif obj.name.lower() == obj_name:
                return obj
    return None


def get_net_objs(host, object_type, refresh=False):
    """
    Retrieves all network objects of the specified type from the host
    :param host: vim.HostSystem
    :param object_type: Type of object to get: (portgroup | vswitch | proxyswitch | vnic | pnic)
    :param refresh: Refresh the host's network system information [default: False]
    :return: list of the network objects
    """
    if refresh:
        host.configManager.networkSystem.RefreshNetworkSystem()  # Pick up any recent changes

    net_type = object_type.lower()
    if net_type == "portgroup":
        objects = host.configManager.networkSystem.networkInfo.portgroup
    elif net_type == "vswitch":
        objects = host.configManager.networkSystem.networkInfo.vswitch
    elif net_type == "proxyswitch":
        objects = host.configManager.networkSystem.networkInfo.proxySwitch
    elif net_type == "vnic ":
        objects = host.configManager.networkSystem.networkInfo.vnic
    elif net_type == "pnic ":
        objects = host.configManager.networkSystem.networkInfo.pnic
    else:
        logging.error("Invalid type %s for get_net_objs", object_type)
        return None
    return list(objects)


def create_vswitch(name, host, num_ports=512):
    """
    Creates a vSwitch
    :param name: Name of the vSwitch to create
    :param host: vim.HostSystem to create the vSwitch on
    :param num_ports: Number of ports the vSwitch should have [default: 512]
    """
    logging.info("Creating vSwitch %s with %d ports on host %s", name, num_ports, host.name)

    vswitch_spec = vim.host.VirtualSwitch.Specification()
    vswitch_spec.numPorts = int(num_ports)

    try:
        host.configManager.networkSystem.AddVirtualSwitch(name, vswitch_spec)
    except vim.fault.AlreadyExists:
        logging.error("vSwitch %s already exists on host %s", name, host.name)

# TODO: edit vswitch


def create_portgroup(name, host, vswitch_name, vlan=0, promiscuous=False):
    """
    Creates a portgroup
    :param name: Name of portgroup to create
    :param host: vim.HostSystem on which to create the port group
    :param vswitch_name: Name of vSwitch on which to create the port group
    :param vlan: VLAN ID of the port group [default: 0]
    :param promiscuous: Put portgroup in promiscuous mode [default: False]
    """
    logging.info("Creating PortGroup %s on vSwitch %s on host %s", name, vswitch_name, host.name)
    logging.debug("VLAN ID: %d \t Promiscuous: %s", vlan, promiscuous)

    spec = vim.host.PortGroup.Specification()
    spec.name = name
    spec.vlanId = int(vlan)
    spec.vswitchName = vswitch_name
    policy = vim.host.NetworkPolicy()
    policy.security = vim.host.NetworkPolicy.SecurityPolicy()
    policy.security.allowPromiscuous = bool(promiscuous)
    policy.security.macChanges = False
    policy.security.forgedTransmits = False
    spec.policy = policy

    try:
        host.configManager.networkSystem.AddPortGroup(spec)
    except vim.fault.AlreadyExists:
        logging.error("PortGroup %s already exists on host %s", name, host.name)
    except vim.fault.NotFound:
        logging.error("vSwitch %s does not exist on host %s", vswitch_name, host.name)

# TODO: edit portgroup


def delete_network(name, host, network_type):
    """
    Deletes the named network from the host
    :param name: Name of the vSwitch to delete
    :param host: vim.HostSystem to delete network from
    :param network_type: Type of the network to remove ("vswitch" | "portgroup")
    """
    logging.info("Deleting %s '%s' from host '%s'", network_type, name, host.name)

    try:
        if network_type.lower() == "vswitch":
            host.configManager.networkSystem.RemoveVirtualSwitch(name)
        elif network_type.lower() == "portgroup":
            host.configManager.networkSystem.RemovePortGroup(name)
    except vim.fault.NotFound:
        logging.error("Tried to remove %s '%s' that does not exist from host '%s'",
                      network_type, name, host.name)
    except vim.fault.ResourceInUse:
        logging.error("%s '%s' can't be removed because there are vNICs associated with it",
                      network_type, name)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from pyVim.connect import SmartConnect, SmartConnectNoSSL, Disconnect
from pyVmomi import vim, vmodl

from adles.vsphere.folder_utils import create_folder, get_in_folder, find_in_folder


class Vsphere:
    """ Maintains connection, logging, and constants for a vSphere instance """

    __version__ = "0.9.8"

    def __init__(self, username=None, password=None, hostname=None,
                 datacenter=None, datastore=None,
                 port=443, use_ssl=False):
        """
        Connects to a vCenter server and initializes a class instance.
        :param username: Username of account to login with [default: prompt user]
        :param password: Password of account to login with [default: prompt user]
        :param hostname: DNS hostname or IP address of vCenter instance [default: prompt user]
        :param datastore: Name of datastore to use [default: first datastore found on server]
        :param datacenter: Name of datacenter to use [default: First datacenter found on server]
        :param port: Port used to connect to vCenter instance [default: 443]
        :param use_ssl: If SSL should be used to connect [default: False]
        """
        self._log = logging.getLogger('Vsphere')
        self._log.debug("Initializing Vsphere %s\nDatacenter: %s\nDatastore: %s\nSSL: %s",
                        Vsphere.__version__, datacenter, datastore, str(use_ssl))
        if username is None:
            username = str(input("Enter username for vSphere: "))
        if password is None:
            from getpass import getpass
            password = str(getpass("Enter password for %s: " % username))
        if hostname is None:
            hostname = str(input("Enter hostname for vSphere: "))
        try:
            self._log.info("Connecting to vSphere: %s@%s:%d", username, hostname, port)
            if use_ssl:  # Connect to server using SSL certificate verification
                self.server = SmartConnect(host=hostname, user=username, pwd=password, port=port)
            else:
                self.server = SmartConnectNoSSL(host=hostname, user=username, pwd=password,
                                                port=port)
        except vim.fault.InvalidLogin:
            self._log.error("Invalid vSphere login credentials for user %s", username)
            exit(1)
        except Exception as e:
            self._log.error("An exception occurred while trying to connect to vSphere: %s", str(e))
            exit(1)

        from atexit import register
        register(Disconnect, self.server)  # Ensures connection to server is closed on program exit

        self._log.info("Connected to vSphere host %s:%d", hostname, port)
        self._log.debug("Current server time: %s", str(self.server.CurrentTime()))

        self.username = username
        self.hostname = hostname
        self.port = port

        self.content = self.server.RetrieveContent()
        self.auth = self.content.authorizationManager
        self.user_dir = self.content.userDirectory

        self.datacenter = self.get_item(vim.Datacenter, name=datacenter)
        if not self.datacenter:
            self._log.error("Could not find a datacenter to initialize with!")
            exit(1)

        self.datastore = self.get_datastore(datastore)
        if not self.datastore:
            self._log.error("Could not find a datastore to initialize with!")
            exit(1)

        self._log.debug("Finished initializing vSphere")

    # From: create_folder_in_datacenter.py in pyvmomi-community-samples
    def create_folder(self, folder_name, create_in=None):
        """
        Creates a VM folder in the specified folder
        :param folder_name: Name of folder to create
        :param create_in: Name of folder or vim.Folder object to create folder in
        [default: root folder of datacenter]
        :return: The created vim.Folder object
        """
        if create_in:
            if type(create_in) is str:  # create_in is a string, so we look it up on the server
                self._log.debug("Retrieving parent folder '%s' from server", create_in)
                parent = self.get_folder(folder_name=create_in)
            else:
                parent = create_in  # create_in is a vim.Folder object, so we just assign it
        else:
            parent = self.content.rootFolder  # Default to using the server root folder
        return create_folder(folder=parent, folder_name=folder_name)

    def gen_clone_spec(self, datastore_name=None, pool_name=None):
        """
        Generates a clone specification used to clone a VM
        :param datastore_name: Name of datastore to put clone on [default: class-defined datastore]
        :param pool_name: Name of resource pool to use for the clone [default: first pool found]
        :return: vim.vm.CloneSpec object
        """
        if datastore_name:
            datastore = self.get_datastore(datastore_name)
        else:
            datastore = self.datastore
        relospec = vim.vm.RelocateSpec()
        relospec.pool = self.get_pool(pool_name)
        relospec.datastore = datastore

        clonespec = vim.vm.CloneSpec()
        clonespec.location = relospec
        return clonespec

    def set_motd(self, message):
        """
        Sets vCenter server Message of the Day (MOTD)
        :param message: Message to set
        """
        self._log.info("Setting vCenter MOTD to %s", message)
        self.content.sessionManager.UpdateServiceMessage(message=message)

    def set_entity_permissions(self, entity, permission):
        """
        Defines or updates rule(s) for the given user or group on the entity
        :param entity: vim.ManagedEntity, The entity on which to set permissions
        :param permission: vim.AuthorizationManager.Permission
        """
        try:
            self.auth.SetEntityPermissions(entity=entity, permission=permission)
        except vim.fault.UserNotFound as e:
            self._log.error("Could not find user '%s' to set permission '%s' on entity '%s'",
                            e.principal, str(permission), entity.name)
        except vim.fault.NotFound:
            self._log.error("Invalid role ID for permission '%s'", str(permission))
        except vmodl.fault.ManagedObjectNotFound as e:
            self._log.error("Entity '%s' does not exist to set permission on", str(e.obj))
        except vim.fault.NoPermission as e:
            self._log.error("Could not set permissions for entity '%s': the current session "
                            "does not have privilege '%s' to set permissions for the entity '%s'",
                            entity.name, e.privilegeId, e.object)
        except vmodl.fault.InvalidArgument as e:
            self._log.error("Invalid argument to set permission '%s' on entity '%s': %s",
                            entity.name, str(permission), str(e))
        except Exception as e:
            self._log.error("Unknown error while setting permissions for entity '%s': %s",
                            entity.name, str(e))

    def get_entity_permissions(self, entity, inherited=True):
        """
        Gets permissions defined on or effective on a managed entity
        :param entity: vim.ManagedEntity
        :param inherited: Include propagating permissions defined by parent entities [default: True]
        :return: vim.AuthorizationManager.Permission
        """
        try:
            return self.auth.RetrieveEntityPermissions(entity=entity, inherited=inherited)
        except vmodl.fault.ManagedObjectNotFound as e:
            self._log.error("Could not find entity '%s' to get permissions from", str(e.obj))
            return None

    def get_role_permissions(self, role_id):
        """
        Gets all permissions that use a particular role
        :param role_id: ID of the role
        :return: vim.AuthorizationManager.Permission
        """
        try:
            return self.auth.RetrieveRolePermissions(roleId=int(role_id))
        except vim.fault.NotFound:
            self._log.error("Role ID %d does not exist", int(role_id))
            return None

    def get_users(self, search='', domain='', exact=False,
                  belong_to_group=None, have_user=None,
                  find_users=True, find_groups=False):
        """
        Returns a list of the users and groups defined for the server.
        NOTE: You must hold the Authorization.ModifyPermissions privilege to invoke this method!
        :param search: Case insensitive substring used to filter results [default: all users]
        :param domain: Domain to be searched [default: local machine]
        :param exact: Search should match user/group name exactly [default: False]
        :param belong_to_group: Only find users/groups that directly belong
        to this group [default: None]
        :param have_user: Only find groups that directly contain this user [default: None]
        :param find_users: If users should be included in the results [default: True]
        :param find_groups: If groups should be included in the results [default: False]
        :return: List of vim.UserSearchResult
        """
        # See for reference: pyvmomi/docs/vim/UserDirectory.rst
        kwargs = {"searchStr": search, "exactMatch": exact,
                  "findUsers": find_users, "findGroups": find_groups}
        if domain != '':
            kwargs["domain"] = domain
        if belong_to_group is not None:
            kwargs["belongsToGroup"] = belong_to_group
        if have_user is not None:
            kwargs["belongsToUser"] = have_user

        try:
            return self.user_dir.RetrieveUserGroups(**kwargs)
        except vim.fault.NotFound:
            self._log.debug("Could not find domain, group or user in call to get_users"
                            "\nkwargs: %s", str(kwargs))
            return None
        except vmodl.fault.NotSupported:
            self._log.error("System doesn't support domains or by-membership queries for get_users")
            return None

    def get_server_info(self):
        """
        Retrieves and formats basic information about the vSphere instance
        :return: string with formatted server information
        """
        about = self.content.about
        info_string = "\n"
        info_string += "Host address: %s:%d\n" % (self.hostname, self.port)
        info_string += "Datacenter  : %s\n" % self.datacenter.name
        info_string += "Datastore   : %s\n" % self.datastore.name
        info_string += "Full name   : %s\n" % about.fullName
        info_string += "Vendor      : %s\n" % about.vendor
        info_string += "Version     : %s\n" % about.version
        info_string += "API type    : %s\n" % about.apiType
        info_string += "API version : %s\n" % about.apiVersion
        info_string += "OS type     : %s" % about.osType
        return info_string

    def get_folder(self, folder_name=None):
        """
        Finds and returns the named Folder
        :param folder_name: Name of the folder [default: Datacenter vmFolder]
        :return: vim.Folder object
        """
        if folder_name:  # Try to find the named folder in the datacenter
            return self.get_obj(self.datacenter, [vim.Folder], folder_name)
        else:  # Default to the VM folder in the datacenter
            self._log.debug("Could not find folder '%s' in Datacenter '%s', defaulting to vmFolder",
                            folder_name, self.datacenter.name)
            return self.datacenter.vmFolder  # Reference: pyvmomi/docs/vim/Datacenter.rst

    def get_vm(self, vm_name):
        """
        Finds and returns the named VM
        :param vm_name: Name of the VM
        :return: vim.VirtualMachine object
        """
        return self.get_item(vim.VirtualMachine, vm_name)

    def get_network(self, network_name, distributed=False):
        """
        Finds and returns the named Network
        :param network_name: Name of the Network
        :param distributed: If the Network is a Distributed PortGroup [default: False]
        :return: vim.Network or vim.dvs.DistributedVirtualPortgroup object
        """
        if not distributed:
            return find_in_folder(folder=self.datacenter.networkFolder, name=network_name,
                                  recursive=True, vimtype=vim.Network)
        else:
            return self.get_item(vim.dvs.DistributedVirtualPortgroup,  network_name)

    def get_host(self, host_name=None):
        """
        Finds and returns the named Host System
        :param host_name: Name of the host [default: first host found in datacenter]
        :return: vim.HostSystem object
        """
        return self.get_item(vim.HostSystem, host_name)

    def get_cluster(self, cluster_name=None):
        """
        Finds and returns the named Cluster
        :param cluster_name: Name of the cluster [default: first cluster found in datacenter]
        :return: vim.ClusterComputeResource object
        """
        return self.get_item(cluster_name, vim.ClusterComputeResource)

    def get_clusters(self):
        """
        Get all the clusters associated with the vCenter server
        :return: List of vim.ClusterComputeResource objects
        """
        return self.get_objs(self.content.rootFolder, [vim.ClusterComputeResource])

    def get_datastore(self, datastore_name=None):
        """
        Finds and returns the named Datastore
        :param datastore_name: Name of the datastore [default: first datastore found in datacenter]
        :return: vim.Datastore object
        """
        return get_in_folder(self.datacenter.datastoreFolder, datastore_name)

    def get_pool(self, pool_name=None):
        """
        Finds and returns the named Resource Pool
        :param pool_name: Name of the resource pool [default: first pool found in datacenter]
        :return: vim.ResourcePool object
        """
        return self.get_item(vim.ResourcePool, pool_name)

    def get_all_vms(self):
        """
        Finds and returns all VMs registered in the Datacenter
        :return: List of vim.VirtualMachine objects
        """
        return self.get_objs(self.datacenter.vmFolder, [vim.VirtualMachine])

    def get_obj(self, container, vimtypes, name, recursive=True):
        """
        Finds and returns named vSphere object of specified type
        :param vimtypes: List of vimtype objects to look for
        :param name: Name of the object
        :param container: Container to search in
        :param recursive: Recursively descend or only look in the current level [default: True]
        :return: The vimtype object found with the specified name, or None if no object was found
        """
        con_view = self.content.viewManager.CreateContainerView(container, vimtypes, recursive)
        obj = None
        for c in con_view.view:
            if c.name.lower() == name.lower():
                obj = c
                break
        con_view.Destroy()
        return obj

    # From: https://github.com/sijis/pyvmomi-examples/vmutils.py
    def get_objs(self, container, vimtypes, recursive=True):
        """
        Get all the vSphere objects associated with a given type
        :param vimtypes: Object to search for
        :param container: Container to search in
        :param recursive: Recursively descend or only look in the current level [default: True]
        :return: List of all vimtype objects found, or None if none were found
        """
        objs = []
        con_view = self.content.viewManager.CreateContainerView(container, vimtypes, recursive)
        for c in con_view.view:
            objs.append(c)
        con_view.Destroy()
        return objs

    def get_item(self, vimtype, name=None, container=None, recursive=True):
        """
        Get a item of specified name and type. Intended to be simple version of get_obj()
        :param vimtype: Type of item
        :param name: Name of item [default: None]
        :param container: Container to search in [default: vCenter server content root folder]
        :param recursive: Recursively descend or only look in the current level [default: True]
        :return: The item found
        """
        contain = (self.content.rootFolder if not container else container)
        if not name:
            return self.get_objs(contain, [vimtype], recursive)[0]
        else:
            return self.get_obj(contain, [vimtype], name, recursive)

    def map_items(self, vimtypes, func, name=None, container=None, recursive=True):
        """
        Apply a function to item(s)
        :param vimtypes: List of vimtype objects to look for
        :param func: Function to apply
        :param name: Name of item to apply to [default: None]
        :param container: Container to search in [default: content.rootFolder]
        :param recursive: Recursively descend or only look in the current level [default: True]
        :return: List of values returned from the function call(s)
        """
        contain = (self.content.rootFolder if not container else container)
        con_view = self.content.viewManager.CreateContainerView(contain, vimtypes, recursive)
        returns = []
        for item in con_view.view:
            if name:
                if hasattr(item, 'name') and item.name.lower() == name.lower():
                    returns.append(func(item))
            else:
                returns.append(func(item))
        con_view.Destroy()
        return returns

    def find_by_uuid(self, uuid, instance_uuid=True):
        """
        Find a VM in the datacenter with the given Instance or BIOS UUID
        :param uuid: UUID to search for (Instance or BIOS for VMs)
        :param instance_uuid: Search for VM Instance UUIDs, otherwise BIOS UUIDs [default: True]
        :return: vim.ManagedEntity
        """
        return self.content.searchIndex.FindByUuid(datacenter=self.datacenter,
                                                   uuid=str(uuid), vmSearch=True,
                                                   instanceUuid=instance_uuid)

    def find_by_ds_path(self, path):
        """
        Finds a VM by it's location on a Datastore
        :param path: Path to the VM's .vmx file on the Datastore
        :return: vim.VirtualMachine
        """
        try:
            return self.content.searchIndex.FindByDatastorePath(datacenter=self.datacenter,
                                                                path=str(path))
        except vim.fault.InvalidDatastore:
            self._log.error("Invalid datastore in path: %s", str(path))
            return None

    def find_by_ip(self, ip, vm_search=True):
        """
        Find a VM or Host using a IP address
        :param ip: IP address string as returned by VMware Tools ipAddress
        :param vm_search: Search for VMs if True, Hosts if False [default: True]
        :return: vim.ManagedEntity
        """
        return self.content.searchIndex.FindByIp(datacenter=self.datacenter,
                                                 ip=str(ip), vmSearch=vm_search)

    def find_by_hostname(self, hostname, vm_search=True):
        """
        Find a VM or Host using a fully-qualified domain name
        :param hostname: Fully-qualified domain name
        :param vm_search: Search for VMs if True, Hosts if False [default: True]
        :return: vim.ManagedEntity
        """
        return self.content.searchIndex.FindByDnsName(datacenter=self.datacenter,
                                                      dnsName=hostname, vmSearch=vm_search)

    def find_by_inv_path(self, path):
        """
        Finds a vim.ManagedEntity (VM, host, resource pool, folder, etc) in a inventory
        :param path: Path to the entity
        :return: vim.ManagedEntity
        """
        return self.content.searchIndex.FindByInventoryPath(inventoryPath=str(path))

    def __repr__(self):
        return "vSphere(%s, %s, %s:%s)" % (self.datacenter.name, self.datastore.name,
                                           self.hostname, self.port)

    def __str__(self):
        return str(self.get_server_info())

    def __hash__(self):
        return hash((self.hostname, self.port, self.username))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.hostname == other.hostname \
               and self.port == other.port and self.username == other.username

    def __ne__(self, other):
        return not self.__eq__(other)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from pyVmomi import vim


def wait_for_task(task):
    """
    Waits for a single vCenter task to finish and return it's result
    :param task: vim.Task object of the task to wait for
    :return: Task result information
    """
    if not task:
        logging.debug("No task was specified to wait for")
        return None
    while True:  # TODO: set a timeout, perhaps based on type of task
        if task.info.state == 'success':
            return task.info.result
        elif task.info.state == 'error':
            # TODO: parse error messages to generate more meaningful errors or warnings
            #   Examples: "already exists", "Cancelled", "permission error" or whatnot
            logging.error("There was an error while completing a task: '%s'",
                          task.info.error.msg)
            return None


# From: list_dc_datastore_info in pyvmomi-community-samples
def get_datastore_info(ds_obj):
    """
    Gets a human-readable summary of a Datastore
    :param ds_obj: vim.Datastore
    :return: String with datastore information
    """
    if not ds_obj:
        logging.error("No Datastore was given to get_datastore_info")
        return None
    from adles.utils import sizeof_fmt
    info_string = "\n"
    summary = ds_obj.summary
    ds_capacity = summary.capacity
    ds_freespace = summary.freeSpace
    ds_uncommitted = summary.uncommitted if summary.uncommitted else 0
    ds_provisioned = ds_capacity - ds_freespace + ds_uncommitted
    ds_overp = ds_provisioned - ds_capacity
    ds_overp_pct = (ds_overp * 100) / ds_capacity if ds_capacity else 0

    info_string += "Name                  : %s\n" % summary.name
    info_string += "URL                   : %s\n" % summary.url
    info_string += "Capacity              : %s\n" % sizeof_fmt(ds_capacity)
    info_string += "Free Space            : %s\n" % sizeof_fmt(ds_freespace)
    info_string += "Uncommitted           : %s\n" % sizeof_fmt(ds_uncommitted)
    info_string += "Provisioned           : %s\n" % sizeof_fmt(ds_provisioned)
    if ds_overp > 0:
        info_string += "Over-provisioned      : %s / %s %%\n" \
                       % (sizeof_fmt(ds_overp), ds_overp_pct)
    info_string += "Hosts                 : %d\n" % len(ds_obj.host)
    info_string += "Virtual Machines      : %d" % len(ds_obj.vm)
    return info_string


def is_folder(obj):
    """
    Checks if object is a vim.Folder
    :param obj: object to check
    :return: Bool
    """
    return hasattr(obj, "childEntity")


def is_vm(obj):
    """
    Checks if object is a vim.VirtualMachine
    :param obj: object to check
    :return: Bool
    """
    return hasattr(obj, "summary")


def is_vnic(device):
    """
    Checks if the device is a VirtualEthernetCard
    :param device: device to check
    :return: Bool
    """
    return isinstance(device, vim.vm.device.VirtualEthernetCard)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from pyVmomi import vim

import adles.utils as utils


class Host:
    """ Represents an ESXi host in a VMware vSphere environment. """
    __version__ = "0.1.0"

    def __init__(self, host):
        self.host = host
        self.name = str(host.name)

    def get_net_item(self, object_type, name):
        """
        Retrieves a network object of the specified type and name from a host
        :param object_type: Type of object to get: (portgroup | vswitch | proxyswitch | vnic | pnic)
        :param name: Name of network object [default: first object found]
        :return: The network object
        """
        if name:
            return self.get_net_obj(object_type, name)
        else:
            return self.get_net_objs(object_type)[0]

    def get_net_obj(self, object_type, name, refresh=False):
        """
        Retrieves a network object of the specified type and name from a host
        :param object_type: Type of object to get: (portgroup | vswitch | proxyswitch | vnic | pnic)
        :param name: Name of network object
        :param refresh: Refresh the host's network system information [default: False]
        :return: The network object
        """
        objs = self.get_net_objs(object_type=object_type, refresh=refresh)
        obj_name = name.lower()
        if objs is not None:
            for obj in objs:
                if object_type == "portgroup" or object_type == "proxyswitch":
                    if obj.spec.name.lower() == obj_name:
                        return obj
                elif object_type == "pnic" or object_type == "vnic":
                    if obj.device.lower() == obj_name:
                        return obj
                elif obj.name.lower() == obj_name:
                    return obj
        return None

    def get_net_objs(self, object_type, refresh=False):
        """
        Retrieves all network objects of the specified type from the host
        :param object_type: Type of object to get: (portgroup | vswitch | proxyswitch | vnic | pnic)
        :param refresh: Refresh the host's network system information [default: False]
        :return: list of the network objects
        """
        if refresh:
            self.host.configManager.networkSystem.RefreshNetworkSystem()  # Pick up recent changes

        net_type = object_type.lower()
        if net_type == "portgroup":
            objects = self.host.configManager.networkSystem.networkInfo.portgroup
        elif net_type == "vswitch":
            objects = self.host.configManager.networkSystem.networkInfo.vswitch
        elif net_type == "proxyswitch":
            objects = self.host.configManager.networkSystem.networkInfo.proxySwitch
        elif net_type == "vnic ":
            objects = self.host.configManager.networkSystem.networkInfo.vnic
        elif net_type == "pnic ":
            objects = self.host.configManager.networkSystem.networkInfo.pnic
        else:
            logging.error("Invalid type %s for get_net_objs", object_type)
            return None
        return list(objects)

    def create_vswitch(self, name, num_ports=512):
        """
        Creates a vSwitch
        :param name: Name of the vSwitch to create
        :param num_ports: Number of ports the vSwitch should have [default: 512]
        """
        logging.info("Creating vSwitch %s with %d ports on host %s", name, num_ports, self.name)

        vswitch_spec = vim.host.VirtualSwitch.Specification()
        vswitch_spec.numPorts = int(num_ports)

        try:
            self.host.configManager.networkSystem.AddVirtualSwitch(name, vswitch_spec)
        except vim.fault.AlreadyExists:
            logging.error("vSwitch %s already exists on host %s", name, self.name)

    def create_portgroup(self, name, vswitch_name, vlan=0, promiscuous=False):
        """
        Creates a portgroup
        :param name: Name of portgroup to create
        :param vswitch_name: Name of vSwitch on which to create the port group
        :param vlan: VLAN ID of the port group [default: 0]
        :param promiscuous: Put portgroup in promiscuous mode [default: False]
        """
        logging.info("Creating PortGroup %s on vSwitch %s on host %s", name, vswitch_name,
                     self.name)
        logging.debug("VLAN ID: %d \t Promiscuous: %s", vlan, promiscuous)

        spec = vim.host.PortGroup.Specification()
        spec.name = name
        spec.vlanId = int(vlan)
        spec.vswitchName = vswitch_name
        policy = vim.host.NetworkPolicy()
        policy.security = vim.host.NetworkPolicy.SecurityPolicy()
        policy.security.allowPromiscuous = bool(promiscuous)
        policy.security.macChanges = False
        policy.security.forgedTransmits = False
        spec.policy = policy

        try:
            self.host.configManager.networkSystem.AddPortGroup(spec)
        except vim.fault.AlreadyExists:
            logging.error("PortGroup %s already exists on host %s", name, self.name)
        except vim.fault.NotFound:
            logging.error("vSwitch %s does not exist on host %s", vswitch_name, self.name)

    def delete_network(self, name, network_type):
        """
        Deletes the named network from the host
        :param name: Name of the vSwitch to delete
        :param network_type: Type of the network to remove ("vswitch" | "portgroup")
        """
        logging.info("Deleting %s '%s' from host '%s'", network_type, name, self.name)

        try:
            if network_type.lower() == "vswitch":
                self.host.configManager.networkSystem.RemoveVirtualSwitch(name)
            elif network_type.lower() == "portgroup":
                self.host.configManager.networkSystem.RemovePortGroup(name)
        except vim.fault.NotFound:
            logging.error("Tried to remove %s '%s' that does not exist from host '%s'",
                          network_type, name, self.name)
        except vim.fault.ResourceInUse:
            logging.error("%s '%s' can't be removed because there are vNICs associated with it",
                          network_type, name)


class Cluster:
    """ Represents a cluster of ESXi hosts in a VMware vSphere environment. """
    __version__ = "0.1.0"

    def __init__(self):
        pass





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
from adles import __version__, __email__, __author__, __url__, __license__


with open('requirements.txt') as f:
    required = f.read().splitlines()

with open('README.rst') as f:
    long_description = f.read()


setup(
    name='ADLES',
    version=__version__,
    packages=find_packages(exclude=['test']) + ['specifications', 'examples'],
    include_package_data=True,
    zip_safe=False,
    install_requires=required,
    entry_points={
        'console_scripts': [
            'adles = adles.scripts.adles_main:main',
            'clone-vms = adles.scripts.clone_vms:main',
            'cleanup-vms = adles.scripts.cleanup_vms:main',
            'vm-power = adles.scripts.vm_power:main',
            'vsphere-info = adles.scripts.vsphere_info:main'
        ]
    },
    author=__author__,
    author_email=__email__,
    description='Automated Deployment of Lab Environments System (ADLES)',
    long_description=long_description,
    url=__url__,
    download_url='https://pypi.python.org/pypi/ADLES',
    license=__license__,
    keywords="adles virtualization automation vmware vsphere yaml labs virtual vms python pyvmomi "
             "cybersecurity education uidaho radicl environments deployment docker lol 1337 setup",
    setup_requires=[
        "pytest-runner==2.11.1"
    ],
    tests_require=[
        "pytest==3.0.7",
        "pytest-cov==2.4.0"
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Intended Audience :: Education',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Education',
        'Topic :: Education :: Testing',
        'Topic :: Security',
        'Topic :: Software Development',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
        'Natural Language :: English'
    ]
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# http://multivax.com/last_question.html

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ADLES: Automated Deployment of Lab Environments System.
Uses formal YAML specifications to create virtual environments for educational purposes.

Usage:
    adles [options]
    adles [options] -c SPEC [-t TYPE]
    adles [options] (-m | -d) [-p] -s SPEC
    adles [options] (--cleanup-masters | --cleanup-enviro) [--nets] -s SPEC

Options:
    -n, --no-color              Do not color terminal output
    -v, --verbose               Emit debugging logs to terminal
    -c, --validate SPEC         Validates syntax of an exercise specification
    -t, --type TYPE             Type of specification to validate: exercise, package, infra
    -s, --spec SPEC             Name of a YAML specification file
    -i, --infra FILE            Specifies the infrastructure configuration file to use
    -p, --package               Build environment from package specification
    -m, --masters               Master creation phase of specification
    -d, --deploy                Environment deployment phase of specification
    --cleanup-masters           Cleanup masters created by a specification
    --cleanup-enviro            Cleanup environment created by a specification
    --nets                      Cleanup networks created during either phase
    --print-spec NAME           Prints the named: exercise, package, infrastructure
    --list-examples             Prints the list of examples available
    --print-example NAME        Prints the named example
    -h, --help                  Shows this help
    --version                   Prints current version

Examples:
    adles --list-examples
    adles -c examples/tutorial.yaml
    adles --verbose --masters --spec examples/experiment.yaml
    adles -vds examples/competition.yaml
    adles --cleanup-masters --nets -s examples/competition.yaml

License:    Apache 2.0
Author:     Christopher Goes <goes8945@vandals.uidaho.edu>
Project:    https://github.com/GhostofGoes/ADLES

"""

import logging

from docopt import docopt
from pyVmomi import vim

from adles.interfaces import Interface
from adles.parser import check_syntax, parse_file
from adles.utils import setup_logging
from adles import __version__


def main():
    args = docopt(__doc__, version=__version__, help=True)
    colors = (False if args["--no-color"] else True)
    setup_logging(filename='adles.log', colors=colors, console_verbose=args["--verbose"])

    if args["--spec"]:
        if args["--package"]:
            # TODO: implement basic extraction of environment spec from package spec
            logging.error("Package specifications are not implemented yet")
            exit(1)

        spec = check_syntax(args["--spec"])
        if spec is None:
            logging.error("Syntax check failed")
            exit(1)
        if "name" not in spec["metadata"]:
            from os.path import basename, splitext
            spec["metadata"]["name"] = splitext(basename(args["--spec"]))[0]

        if args["--infra"]:
            from os.path import exists
            infra_file = args["--infra"]
            if not exists(infra_file):
                logging.error("Specified infrastructure config file '%s' could not be found,"
                              " falling back to exercise configuration", infra_file)
            else:
                logging.debug("Overriding infrastructure config file with '%s'", infra_file)
                spec["metadata"]["infra-file"] = infra_file

        try:
            interface = Interface(spec, parse_file(spec["metadata"]["infra-file"]))
            if args["--masters"]:
                interface.create_masters()
                logging.info("Finished creation of Masters for environment %s",
                             spec["metadata"]["name"])
            elif args["--deploy"]:
                interface.deploy_environment()
                logging.info("Finished deployment of environment %s", spec["metadata"]["name"])
            elif args["--cleanup-masters"]:
                interface.cleanup_masters(args["--nets"])
            elif args["--cleanup-enviro"]:
                interface.cleanup_environment(args["--nets"])
            else:
                logging.error("Invalid flags for --spec. Argument dump:\n%s", str(args))
        except vim.fault.NoPermission as e:
            logging.error("Permission error: \n%s", str(e))
            exit(1)
        except KeyboardInterrupt:
            logging.error("User terminated session prematurely")
            exit(1)

    elif args["--validate"]:
        if args["--type"]:
            spec_type = args["--type"]
        else:
            spec_type = "exercise"

        if check_syntax(args["--validate"], spec_type) is None:
            logging.error("Syntax check failed")

    # TODO: run example as input (should do this by supporting stdin)
    elif args["--list-examples"] or args["--print-example"]:
        from pkg_resources import Requirement, resource_filename
        from os import listdir, path
        example_dir = resource_filename(Requirement.parse("ADLES"), "examples")
        examples = [x[:-5] for x in listdir(example_dir) if ".yaml" in x]  # Filter non-yaml
        if args["--list-examples"]:
            print("Example scenarios that can be printed using --print-example <name>")
            print("Name".ljust(25) + "Version".ljust(10) + "Description")  # Print header
            for example in examples:
                metadata = parse_file(path.join(example_dir, example + ".yaml"))["metadata"]
                name = str(example).ljust(25)
                ver = str(metadata["version"]).ljust(10)
                desc = str(metadata["description"])
                print(name + ver + desc)
        else:
            example = args["--print-example"]
            if example in examples:
                with open(path.join(example_dir, example + ".yaml")) as f:
                    print(f.read())
            else:
                logging.error("Invalid example: %s", example)

    elif args["--print-spec"]:
        from pkg_resources import Requirement, resource_filename
        from os.path import join
        spec = args["--print-spec"]
        specs = ["exercise", "package", "infrastructure"]

        if spec in specs:
            filename = resource_filename(Requirement.parse("ADLES"),
                                         join("specifications", spec + "-specification.yaml"))
            with open(filename) as f:
                print(f.read())
        else:
            logging.error("Invalid specification: %s", spec)

    else:
        logging.error("Invalid arguments. Argument dump:\n%s", str(args))

if __name__ == '__main__':
    main()
